/*=========================================================================

  Program: GDCM (Grassroots DICOM). A DICOM library

  Copyright (c) 2006-2011 Mathieu Malaterre
  All rights reserved.
  See Copyright.txt or http://gdcm.sourceforge.net/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef GDCMENCAPSULATEDDOCUMENT_H
#define GDCMENCAPSULATEDDOCUMENT_H

#include "gdcmFile.h"

namespace gdcm
{
/**
 * \brief EncapsulatedDocument
 */
class GDCM_EXPORT EncapsulatedDocument
{
public:
  EncapsulatedDocument() = default;

private:
};

} // end namespace gdcm

#endif //GDCMENCAPSULATEDDOCUMENT_H
